import { ApolloCache } from '@apollo/client/core';
export declare type LogLevel = 'log' | 'warn' | 'error';
export declare type LogLine = [LogLevel, any[]];
export declare type TriggerUninstallFunction = () => void;
export declare type TriggerFunction = (persist: () => void) => TriggerUninstallFunction;
export declare type PersistenceMapperFunction = (data: any) => Promise<any>;
export declare type PersistedData<T> = T | string | null;
export interface PersistentStorage<T> {
    getItem: (key: string) => Promise<T | null> | T | null;
    setItem: (key: string, value: T) => Promise<T> | Promise<void> | void | T;
    removeItem: (key: string) => Promise<T> | Promise<void> | void;
}
declare type StorageType<T, TSerialize extends boolean> = TSerialize extends true ? PersistentStorage<string> : PersistentStorage<T>;
export interface ApolloPersistOptions<TSerialized, TSerialize extends boolean = true> {
    cache: ApolloCache<TSerialized>;
    storage: StorageType<PersistedData<TSerialized>, TSerialize>;
    trigger?: 'write' | 'background' | TriggerFunction | false;
    debounce?: number;
    key?: string;
    serialize?: TSerialize;
    maxSize?: number | false;
    persistenceMapper?: PersistenceMapperFunction;
    debug?: boolean;
}
export {};
